	
// MacCapture.h : CMacCapture ̐錾

#ifndef __MACCAPTURE_H_
#define __MACCAPTURE_H_

#include "resource.h"       // C V{
#include "NCap.h"
#include "MacCapCP.h"

const UINT WM_STOP_READ = WM_APP + 1;
const UINT WM_START_READ = WM_APP + 2;

LPBYTE Pack(LPVOID, LPCOLESTR, int);
LPOLESTR Unpack(LPOLESTR, LPVOID, int);

class CFilter {
public:
	enum ETOpt {
		NO_OPT = 0,
		IP_ONLY = 1,
		IP_ARP = 2,
	};
	CFilter(const CFilter& f)
	{
		m_opt = f.m_opt;
		m_cb = f.m_cb;
		m_offset = f.m_offset;
		m_dwVal = f.m_dwVal;
		if (f.m_pData)
		{
			m_pData = new BYTE[m_cb];
			CopyMemory(m_pData, f.m_pData, m_cb);
		}
		else
			m_pData = NULL;
	}
	CFilter() : m_pData(NULL), m_cb(0), m_offset(0), m_dwVal(0), m_opt(NO_OPT)
	{
	}
	CFilter(LPBYTE pb, int cb, int off, ETOpt e = NO_OPT)
		: m_dwVal(0), m_pData(NULL), m_opt(e)
	{
		m_cb = cb;
		m_offset = off;
		if (cb <= sizeof(DWORD))
		{
			CopyMemory(&m_dwVal, pb, cb);
		}
		else
		{
			m_pData = new BYTE[cb];
			CopyMemory(m_pData, pb, cb);
		}
	}
	~CFilter()
	{
		delete[] m_pData;
	}
	CFilter& operator=(const CFilter& f)
	{
		delete[] m_pData;
		m_opt = f.m_opt;
		m_cb = f.m_cb;
		m_offset = f.m_offset;
		m_dwVal = f.m_dwVal;
		if (f.m_pData)
		{
			m_pData = new BYTE[m_cb];
			CopyMemory(m_pData, f.m_pData, m_cb);
		}
		else
			m_pData = NULL;
		return *this;
	}
	inline int GetSize() const { return m_cb; }
	bool DataCompare(LPBYTE pb) const;
	inline bool CheckRange(int sz) const 
	{
		return ((m_offset + m_cb) > sz) ? false : true;
	}
private:
	LPBYTE m_pData;
	int m_cb;
	int m_offset;
	DWORD m_dwVal;
	ETOpt m_opt;
};

typedef std::vector<CFilter> AFilter;
typedef std::vector<CFilter>::iterator AFilterIter;

class CAdapters;
/////////////////////////////////////////////////////////////////////////////
// CMacCapture
class ATL_NO_VTABLE CMacCapture : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CMacCapture, &CLSID_MacCapture>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CMacCapture>,
	public IDispatchImpl<IMacCapture, &IID_IMacCapture, &LIBID_MACCAPLib>,
	public IProvideClassInfo2Impl<&CLSID_MacCapture, &DIID__IMacCaptureEvents, &LIBID_MACCAPLib>,
	public CProxy_IMacCaptureEvents< CMacCapture >
{
public:
	CMacCapture() 
		: m_pAdapters(NULL), m_hAdapter(INVALID_HANDLE_VALUE), 
		  m_szAdapter(NULL), m_hThread(NULL), m_pStream(NULL),
		  m_dwBuffSize(8 * 1024 * 1000), m_fDuringCapture(false),
		  m_fRecvDataEnabled(false), m_dwMonitorInterval(500)
	{
		m_pUnkMarshaler = NULL;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_MACCAPTURE)
DECLARE_GET_CONTROLLING_UNKNOWN()

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CMacCapture)
	COM_INTERFACE_ENTRY(IMacCapture)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
	COM_INTERFACE_ENTRY_FUNC(IID_IStream, 0, StreamAgg)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CMacCapture)
CONNECTION_POINT_ENTRY(DIID__IMacCaptureEvents)
END_CONNECTION_POINT_MAP()


	HRESULT FinalConstruct();

	void FinalRelease();

	CComPtr<IUnknown> m_pUnkMarshaler;

	static HRESULT WINAPI StreamAgg(void* pv, REFIID riid, LPVOID* ppv, DWORD dw);

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IMacCapture
public:
	HRESULT STDMETHODCALLTYPE get_MACAddress( 
            /* [retval][out] */ VARIANT __RPC_FAR *pRetVal);
        
	HRESULT STDMETHODCALLTYPE get_Adapters( 
            /* [retval][out] */ IAdapterCollection __RPC_FAR *__RPC_FAR *ppRetVal);
        
	HRESULT STDMETHODCALLTYPE Open( 
            /* [in] */ BSTR AdapterName);
        
	HRESULT STDMETHODCALLTYPE Close( );
        
	HRESULT STDMETHODCALLTYPE Start(
            /* [in] */ VARIANT_BOOL SyncMode);
        
	HRESULT STDMETHODCALLTYPE Stop( );

	HRESULT STDMETHODCALLTYPE put_BufferSize( 
            /* [in] */ long NewVal);
        
	HRESULT STDMETHODCALLTYPE get_BufferSize( 
            /* [retval][out] */ long __RPC_FAR *pRetVal);

	HRESULT STDMETHODCALLTYPE put_RecvDataEnabled( 
            /* [in] */ VARIANT_BOOL NewVal);
        
	HRESULT STDMETHODCALLTYPE get_RecvDataEnabled( 
            /* [retval][out] */ VARIANT_BOOL __RPC_FAR *pRetVal);
        
	HRESULT STDMETHODCALLTYPE put_MonitorEventInterval( 
            /* [in] */ long NewVal);
        
	HRESULT STDMETHODCALLTYPE get_MonitorEventInterval( 
            /* [retval][out] */ long __RPC_FAR *pRetVal);

	HRESULT STDMETHODCALLTYPE SaveData( 
            /* [in] */ BSTR FileName,
            /* [retval][out] */ long __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE LoadData( 
            /* [in] */ BSTR FileName,
            /* [retval][out] */ long __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE get_Packets( 
            /* [retval][out] */ IPacketCollection __RPC_FAR *__RPC_FAR *ppRetVal);

	HRESULT STDMETHODCALLTYPE RemoveFilters( void);
        
	HRESULT STDMETHODCALLTYPE AddFilter( 
            /* [in] */ IFilterParam __RPC_FAR *pPamra);
        
	HRESULT STDMETHODCALLTYPE get_Protocols( 
            /* [retval][out] */ IProtocolNameCollection __RPC_FAR *__RPC_FAR *ppRetVal);
        
	HRESULT STDMETHODCALLTYPE get_EtherTypes( 
            /* [retval][out] */ IEtherTypeNameCollection __RPC_FAR *__RPC_FAR *ppRetVal);

	HRESULT STDMETHODCALLTYPE UnpackHelper( 
            /* [in] */ VARIANT v,
            /* [retval][out] */ BSTR __RPC_FAR *pResult);

	HRESULT STDMETHODCALLTYPE get_Vendors( 
            /* [retval][out] */ IVendorCollection __RPC_FAR *__RPC_FAR *ppRetVal);
private:
	HRESULT CloseHandle();
	CAdapters* m_pAdapters;
	HANDLE m_hAdapter;
	BYTE m_MacAddress[MacAddressSize];
	LPTSTR m_szAdapter;
	DWORD m_dwBuffSize, m_dwPackets, m_dwWrote, m_dwMonitorInterval, m_dwLastWrote;
	bool m_fDuringCapture, m_fRecvDataEnabled;
	struct ThreadParam {
		CMacCapture* pParent;
		HANDLE hSem;
		IStream* pClientSink;
	};
	static void StopCapture(CMacCapture*, HANDLE&, IDispatch*, DWORD);
	static unsigned long WINAPI RecvThreadProc(  LPVOID lpParameter  );
	HRESULT SyncCapture(HGLOBAL gh);

	HANDLE m_hThread;
	DWORD m_idThread;
	LPSTREAM m_pStream;
	AFilter m_afilter;
};

#endif //__MACCAPTURE_H_
