// MacPacket.cpp : CMacPacket ̃Cve[V
#include "stdafx.h"
#include "MacCap.h"
#include "MacPacket.h"

#include "AssNum.h"
/////////////////////////////////////////////////////////////////////////////
// CMacPacket
void CMacPacket::SetParam(IStream* pStream, DWORD dwSeekPoint)
{
	ATLTRACE(_T("Packet Offset = %d\n"), dwSeekPoint);
	m_pStream = pStream;
	BYTE szBuff[PACKET_HEADER_SIZE];
	ULONG ul;
	LARGE_INTEGER dlib;
	dlib.HighPart = 0;
	dlib.LowPart = dwSeekPoint;
	HRESULT hr = pStream->Seek(dlib, STREAM_SEEK_SET, NULL);
	hr = pStream->Read(szBuff, PACKET_HEADER_SIZE, &ul);
	m_dwSize = *(DWORD*)szBuff - PACKET_HEADER_SIZE;
	CopyMemory(&m_ftime, &szBuff[RCVTIME_OFFSET], sizeof(FILETIME));
	m_dwSeekPoint = PACKET_HEADER_SIZE + dwSeekPoint;
}

HRESULT STDMETHODCALLTYPE CMacPacket::get_Value( 
            /* [retval][out] */ VARIANT __RPC_FAR *pData)
{
	if (!pData) return E_POINTER;
	SAFEARRAYBOUND rgsabound[1];
	rgsabound[0].lLbound = 0;
	rgsabound[0].cElements = m_dwSize;
	SAFEARRAY* pAry = SafeArrayCreate(VT_UI1, 1, rgsabound);
	if (pAry == NULL) return E_OUTOFMEMORY;
	SafeArrayLock(pAry);
	ULONG ul;
	LARGE_INTEGER dlib;
	dlib.HighPart = 0;
	dlib.LowPart = m_dwSeekPoint;
	m_pStream->Seek(dlib, STREAM_SEEK_SET, NULL);
	HRESULT hr = m_pStream->Read(pAry->pvData, m_dwSize, &ul);
	if (m_fHeader == false && hr == S_OK)
	{
		m_fHeader = true;
		CopyMemory(m_abData, pAry->pvData, sizeof(m_abData));
	}
	SafeArrayUnlock(pAry);
	if (hr != S_OK) return hr;
	VariantInit(pData);
	pData->vt = VT_ARRAY | VT_UI1;
	pData->parray = pAry;
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_RecvTime( 
            /* [retval][out] */ DATE __RPC_FAR *pDate)
{
	if (!pDate) return E_POINTER;
	SYSTEMTIME stime;
	FILETIME ftime;
	FileTimeToLocalFileTime(&m_ftime, &ftime);
	FileTimeToSystemTime(&ftime, &stime);
	SystemTimeToVariantTime(&stime, pDate);
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CMacPacket::get_EtherTypeString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	*pResult = SysAllocString(GetEtherTypeString(m_abData));
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_IsMulticast( 
            /* [retval][out] */ VARIANT_BOOL __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	*pResult = IsMulticast(m_abData) ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_SrcVendorString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	*pResult = SysAllocString(GetVendorString(&m_abData[SRC_MAC_ADDRESS]));
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_DestVendorString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	LPCOLESTR psz = GetMulticastDescription(&m_abData[DST_MAC_ADDRESS], ntohs(*(WORD*)&m_abData[ETHER_TYPE]));
	if (psz == NULL)
		psz = SysAllocString(GetVendorString(&m_abData[DST_MAC_ADDRESS]));
	*pResult = SysAllocString(psz);
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_FileTime( 
            /* [retval][out] */ CY __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	CopyMemory(pResult, &m_ftime, sizeof(CY));
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_InternetProtocolString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	WORD w = ntohs(*(WORD*)&m_abData[ETHER_TYPE]);
	if (w == ET_IP)
		*pResult = SysAllocString(GetProtoName(m_abData[IPH_PROTOCOL]));
	else
		*pResult = SysAllocString(L"NOT IP");
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_SrcMacString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	OLECHAR szMac[32];
	swprintf(szMac, L"%02X-%02X-%02X-%02X-%02X-%02X",
		m_abData[SRC_MAC_ADDRESS], m_abData[SRC_MAC_ADDRESS + 1], m_abData[SRC_MAC_ADDRESS + 2],
		m_abData[SRC_MAC_ADDRESS + 3], m_abData[SRC_MAC_ADDRESS + 4], m_abData[SRC_MAC_ADDRESS + 5]);
	*pResult = SysAllocString(szMac);
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_DestMacString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	OLECHAR szMac[32];
	swprintf(szMac, L"%02X-%02X-%02X-%02X-%02X-%02X",
		m_abData[DST_MAC_ADDRESS], m_abData[DST_MAC_ADDRESS + 1], m_abData[DST_MAC_ADDRESS + 2],
		m_abData[DST_MAC_ADDRESS + 3], m_abData[DST_MAC_ADDRESS + 4], m_abData[DST_MAC_ADDRESS + 5]);
	*pResult = SysAllocString(szMac);
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_SrcIPAddressString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	WORD w = ntohs(*(WORD*)&m_abData[ETHER_TYPE]);
	if (w == ET_IP)
		GetIPAddress(IPH_SRC, pResult);
	else if (w == ET_ARP || w == ET_RARP)
		GetIPAddress(ARP_SRC_IP, pResult);
	else
		*pResult = SysAllocString(L"No IP Address");
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_DestIPAddressString( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	WORD w = ntohs(*(WORD*)&m_abData[ETHER_TYPE]);
	if (w == ET_IP)
		GetIPAddress(IPH_DST, pResult);
	else if (w == ET_ARP || w == ET_RARP)
		GetIPAddress(ARP_DST_IP, pResult);
	else
		*pResult = SysAllocString(L"No IP Address");
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CMacPacket::get_EtherType( 
            /* [retval][out] */ long __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	*pResult = ntohs(*(WORD*)&m_abData[ETHER_TYPE]);
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_InternetProtocol(
            /* [retval][out] */ short __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	HRESULT hr = SetHeader();
	if (hr != S_OK) return hr;
	*pResult = (ntohs(*(WORD*)&m_abData[ETHER_TYPE]) == ET_IP) ? m_abData[IPH_PROTOCOL] : -1;
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacPacket::get_Length( 
            /* [retval][out] */ long __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	*pResult = m_dwSize;
	return S_OK;
}

///////////////////////////////////////////////////////////////////
STDMETHODIMP CMacPacket::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IMacPacket
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CMacPacket::SetHeader()
{
	if (m_fHeader) return S_OK;
	ULONG ul;
	LARGE_INTEGER dlib;
	dlib.HighPart = 0;
	dlib.LowPart = m_dwSeekPoint;
	m_pStream->Seek(dlib, STREAM_SEEK_SET, NULL);
	HRESULT hr = m_pStream->Read(m_abData, sizeof(m_abData), &ul);
	if (hr == S_OK)
		m_fHeader = true;
	return hr;
}

void CMacPacket::GetIPAddress(int offset, BSTR* pResult)
{
	OLECHAR sz[24];
	swprintf(sz, L"%d.%d.%d.%d", m_abData[offset], m_abData[offset + 1],
		m_abData[offset + 2], m_abData[offset + 3]);
	*pResult = SysAllocString(sz);
}

